@@ -63,7 +63,7 @@ static int next_target(struct dm_target_spec *last, void *begin, void *end,
 
 void dm_error(const char *message)
 {
-	WARN("%s", message);
+	DMWARN("%s", message);
 }
 
 /*
@@ -91,7 +91,7 @@ static int populate_table(struct dm_table *table, struct dm_ioctl *args)
 	offset_t high = 0;
 
 	if (!args->target_count) {
-		WARN("No targets specified");
+		DMWARN("populate_table: no targets specified");
 		return -EINVAL;
 	}
 
@@ -183,6 +183,8 @@ static int create(struct dm_ioctl *param, struct dm_ioctl *user)
 	if ((r = dm_create(param->name, param->minor, t, &md)))
 		goto bad;
 
+	dm_set_ro(md, param->read_only);
+
 	if ((r = info(param->name, user))) {
 		dm_destroy(md);
 		goto bad;
@@ -238,6 +240,8 @@ static int reload(struct dm_ioctl *param)
 		return r;
 	}
 
+	dm_set_ro(md, param->read_only);
+
 	return 0;
 }
 
@@ -267,6 +271,12 @@ static int ctl_ioctl(struct inode *inode, struct file *file,
 	if ((r = copy_params((struct dm_ioctl *) a, &p)))
 		return r;
 
+	if (strcmp(DM_IOCTL_VERSION, p->version)) {
+		DMWARN("dm_ctl_ioctl: struct dm_ioctl version incompatible");
+		r = -EINVAL;
+		goto out;
+	}
+
 	switch (command) {
 	case DM_CREATE:
 		r = create(p, (struct dm_ioctl *) a);
@@ -289,10 +299,11 @@ static int ctl_ioctl(struct inode *inode, struct file *file,
 		break;
 
 	default:
-		WARN("dm_ctl_ioctl: unknown command 0x%x\n", command);
+		DMWARN("dm_ctl_ioctl: unknown command 0x%x", command);
 		r = -EINVAL;
 	}
 
+      out:
 	free_params(p);
 	return r;
 }
@@ -312,7 +323,7 @@ int dm_interface_init(void)
 
 	r = devfs_register_chrdev(DM_CHAR_MAJOR, DM_DIR, &_ctl_fops);
 	if (r < 0) {
-		WARN("devfs_register_chrdev failed for dm control dev");
+		DMERR("devfs_register_chrdev failed for control device");
 		return -EIO;
 	}
 
@@ -329,5 +340,5 @@ void dm_interface_exit(void)
 	// FIXME: remove control device
 
 	if (devfs_unregister_chrdev(DM_CHAR_MAJOR, DM_DIR) < 0)
-		WARN("devfs_unregister_chrdev failed for dm control device");
+		DMERR("devfs_unregister_chrdev failed for control device");
 }
